@@ -1,13 +1,15 @@
-use serde::{Deserialize, Serialize};
-use sqlparser::ast::AlterTableOperation as ASTAlterTableOperation;
+use std::str::FromStr;
 
+use serde::{Deserialize, Serialize};
+
+use crate::error::CommonError;
 use crate::spec::data_type::Schema;
 use crate::spec::expression::{
     CommonInlineUserDefinedFunction, CommonInlineUserDefinedTableFunction, Expr, ObjectName,
     SortOrder,
 };
 use crate::spec::literal::Literal;
-use crate::spec::Identifier;
+use crate::spec::{DataType, FunctionDefinition, Identifier};
 
 /// Unresolved logical plan node for Sail.
 /// As a starting point, the definition matches the structure of the `Relation` message
@@ -40,7 +42,6 @@ pub struct QueryPlan {
     #[serde(flatten)]
     pub node: QueryNode,
     pub plan_id: Option<i64>,
-    pub source_info: Option<String>,
 }
 
 impl QueryPlan {
@@ -48,7 +49,6 @@ impl QueryPlan {
         Self {
             node,
             plan_id: None,
-            source_info: None,
         }
     }
 }
@@ -59,7 +59,6 @@ pub struct CommandPlan {
     #[serde(flatten)]
     pub node: CommandNode,
     pub plan_id: Option<i64>,
-    pub source_info: Option<String>,
 }
 
 impl CommandPlan {
@@ -67,7 +66,6 @@ impl CommandPlan {
         Self {
             node,
             plan_id: None,
-            source_info: None,
         }
     }
 }
@@ -97,8 +95,8 @@ pub enum QueryNode {
     },
     Limit {
         input: Box<QueryPlan>,
-        skip: usize,
-        limit: usize,
+        skip: Option<Expr>,
+        limit: Option<Expr>,
     },
     Aggregate(Aggregate),
     LocalRelation {
@@ -106,10 +104,6 @@ pub enum QueryNode {
         schema: Option<Schema>,
     },
     Sample(Sample),
-    Offset {
-        input: Box<QueryPlan>,
-        offset: usize,
-    },
     Deduplicate(Deduplicate),
     Range(Range),
     SubqueryAlias {
@@ -137,7 +131,7 @@ pub enum QueryNode {
     },
     Tail {
         input: Box<QueryPlan>,
-        limit: usize,
+        limit: Expr,
     },
     WithColumns {
         input: Box<QueryPlan>,
@@ -175,8 +169,6 @@ pub enum QueryNode {
     WithWatermark(WithWatermark),
     ApplyInPandasWithState(ApplyInPandasWithState),
     CachedLocalRelation {
-        user_id: String,
-        session_id: String,
         hash: String,
     },
     CachedRemoteRelation {
@@ -194,7 +186,7 @@ pub enum QueryNode {
         columns: Vec<Identifier>,
         min_non_nulls: Option<usize>,
     },
-    ReplaceNa {
+    Replace {
         input: Box<QueryPlan>,
         columns: Vec<Identifier>,
         replacements: Vec<Replacement>,
@@ -247,8 +239,8 @@ pub enum QueryNode {
     },
     WithParameters {
         input: Box<QueryPlan>,
-        positional_arguments: Vec<Literal>,
-        named_arguments: Vec<(String, Literal)>,
+        positional_arguments: Vec<Expr>,
+        named_arguments: Vec<(String, Expr)>,
     },
     Values(Vec<Vec<Expr>>),
     TableAlias {
@@ -261,6 +253,15 @@ pub enum QueryNode {
         recursive: bool,
         ctes: Vec<(Identifier, QueryPlan)>,
     },
+    LateralView {
+        input: Option<Box<QueryPlan>>,
+        function: ObjectName,
+        arguments: Vec<Expr>,
+        named_arguments: Vec<(Identifier, Expr)>,
+        table_alias: Option<ObjectName>,
+        column_aliases: Option<Vec<Identifier>>,
+        outer: bool,
+    },
 }
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
@@ -273,19 +274,23 @@ pub enum CommandNode {
     // catalog operations
     CurrentDatabase,
     SetCurrentDatabase {
-        database_name: Identifier,
+        database: ObjectName,
     },
     ListDatabases {
-        catalog: Option<Identifier>,
-        database_pattern: Option<String>,
+        qualifier: Option<ObjectName>,
+        pattern: Option<String>,
     },
     ListTables {
         database: Option<ObjectName>,
-        table_pattern: Option<String>,
+        pattern: Option<String>,
+    },
+    ListViews {
+        database: Option<ObjectName>,
+        pattern: Option<String>,
     },
     ListFunctions {
         database: Option<ObjectName>,
-        function_pattern: Option<String>,
+        pattern: Option<String>,
     },
     ListColumns {
         table: ObjectName,
@@ -313,6 +318,13 @@ pub enum CommandNode {
         #[serde(flatten)]
         definition: TableDefinition,
     },
+    CreateTableAsSelect {
+        table: ObjectName,
+        #[serde(flatten)]
+        definition: TableDefinition,
+        /// The query for `CREATE TABLE ... AS SELECT ...` (CTAS) statements.
+        query: Box<QueryPlan>,
+    },
     RecoverPartitions {
         table: ObjectName,
     },
@@ -321,10 +333,13 @@ pub enum CommandNode {
     },
     CacheTable {
         table: ObjectName,
+        lazy: bool,
         storage_level: Option<StorageLevel>,
+        query: Option<Box<QueryPlan>>,
     },
     UncacheTable {
         table: ObjectName,
+        if_exists: bool,
     },
     ClearCache,
     RefreshTable {
@@ -335,10 +350,10 @@ pub enum CommandNode {
     },
     CurrentCatalog,
     SetCurrentCatalog {
-        catalog_name: Identifier,
+        catalog: Identifier,
     },
     ListCatalogs {
-        catalog_pattern: Option<String>,
+        pattern: Option<String>,
     },
     CreateCatalog {
         catalog: Identifier,
@@ -358,6 +373,9 @@ pub enum CommandNode {
     },
     RegisterFunction(CommonInlineUserDefinedFunction),
     RegisterTableFunction(CommonInlineUserDefinedTableFunction),
+    RefreshFunction {
+        function: ObjectName,
+    },
     DropFunction {
         function: ObjectName,
         if_exists: bool,
@@ -373,13 +391,25 @@ pub enum CommandNode {
         #[serde(flatten)]
         definition: ViewDefinition,
     },
+    /// Drop a view or a temporary view.
     DropView {
         view: ObjectName,
-        /// An optional view kind to match the view name.
-        kind: Option<ViewKind>,
+        if_exists: bool,
+    },
+    CreateTemporaryView {
+        view: Identifier,
+        is_global: bool,
+        #[serde(flatten)]
+        definition: TemporaryViewDefinition,
+    },
+    DropTemporaryView {
+        view: Identifier,
+        is_global: bool,
         if_exists: bool,
     },
     Write(Write),
+    WriteTo(WriteTo),
+    WriteStream(WriteStream),
     Explain {
         // TODO: Support stringified_plans
         mode: ExplainMode,
@@ -388,36 +418,109 @@ pub enum CommandNode {
     InsertInto {
         input: Box<QueryPlan>,
         table: ObjectName,
-        columns: Vec<Identifier>,
-        partition_spec: Vec<Expr>,
-        overwrite: bool,
+        mode: InsertMode,
+        partition: Vec<(Identifier, Option<Expr>)>,
+        if_not_exists: bool,
+    },
+    InsertOverwriteDirectory {
+        input: Box<QueryPlan>,
+        local: bool,
+        location: Option<String>,
+        file_format: Option<TableFileFormat>,
+        row_format: Option<TableRowFormat>,
+        options: Vec<(String, String)>,
+    },
+    MergeInto {
+        target: ObjectName,
+        with_schema_evolution: bool,
+        // TODO: add other fields
     },
     SetVariable {
-        variable: Identifier,
+        variable: String,
         value: String,
     },
     Update {
-        input: Box<QueryPlan>,
         table: ObjectName,
         table_alias: Option<Identifier>,
         assignments: Vec<(ObjectName, Expr)>,
+        condition: Option<Expr>,
     },
     Delete {
         table: ObjectName,
+        table_alias: Option<Identifier>,
         condition: Option<Expr>,
     },
     AlterTable {
         table: ObjectName,
         if_exists: bool,
-        // TODO: When we implement a `AlterTable` Extension in DataFusion, create spec equivalent of `AlterTableOperation`
-        operations: Vec<ASTAlterTableOperation>,
-        location: Option<AlterTableLocation>,
+        operation: AlterTableOperation,
+    },
+    AlterView {
+        view: ObjectName,
+        if_exists: bool,
+        operation: AlterViewOperation,
+    },
+    LoadData {
+        local: bool,
+        location: String,
+        table: ObjectName,
+        overwrite: bool,
+        partition: Vec<(Identifier, Option<Expr>)>,
+    },
+    AnalyzeTable {
+        table: ObjectName,
+        partition: Vec<(Identifier, Option<Expr>)>,
+        columns: Vec<ObjectName>,
+        no_scan: bool,
+    },
+    AnalyzeTables {
+        from: Option<ObjectName>,
+        no_scan: bool,
+    },
+    DescribeQuery {
+        query: Box<QueryPlan>,
+    },
+    DescribeFunction {
+        function: ObjectName,
+        extended: bool,
+    },
+    DescribeCatalog {
+        catalog: ObjectName,
+        extended: bool,
+    },
+    DescribeDatabase {
+        database: ObjectName,
+        extended: bool,
+    },
+    DescribeTable {
+        table: ObjectName,
+        extended: bool,
+        partition: Vec<(Identifier, Option<Expr>)>,
+        column: Option<ObjectName>,
+    },
+    CommentOnCatalog {
+        catalog: ObjectName,
+        value: Option<String>,
+    },
+    CommentOnDatabase {
+        database: ObjectName,
+        value: Option<String>,
+    },
+    CommentOnTable {
+        table: ObjectName,
+        value: Option<String>,
+    },
+    CommentOnColumn {
+        column: ObjectName,
+        value: Option<String>,
     },
 }
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase", rename_all_fields = "camelCase")]
+#[allow(clippy::large_enum_variant)]
 pub enum ReadType {
+    // FIXME: Rust 1.87 triggers `clippy::large_enum_variant` warning
     NamedTable(ReadNamedTable),
     Udtf(ReadUdtf),
     DataSource(ReadDataSource),
@@ -427,14 +530,46 @@ pub enum ReadType {
 #[serde(rename_all = "camelCase")]
 pub struct ReadNamedTable {
     pub name: ObjectName,
+    pub temporal: Option<TableTemporal>,
+    pub sample: Option<TableSample>,
     pub options: Vec<(String, String)>,
 }
 
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase", rename_all_fields = "camelCase")]
+pub enum TableTemporal {
+    Version { value: Expr },
+    Timestamp { value: Expr },
+}
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct TableSample {
+    pub method: TableSampleMethod,
+    pub seed: Option<i64>,
+}
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase", rename_all_fields = "camelCase")]
+pub enum TableSampleMethod {
+    Percent {
+        value: Expr,
+    },
+    Rows {
+        value: Expr,
+    },
+    Bucket {
+        numerator: usize,
+        denominator: usize,
+    },
+}
+
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct ReadUdtf {
     pub name: ObjectName,
     pub arguments: Vec<Expr>,
+    pub named_arguments: Vec<(Identifier, Expr)>,
     pub options: Vec<(String, String)>,
 }
 
@@ -453,9 +588,8 @@ pub struct ReadDataSource {
 pub struct Join {
     pub left: Box<QueryPlan>,
     pub right: Box<QueryPlan>,
-    pub join_condition: Option<Expr>,
     pub join_type: JoinType,
-    pub using_columns: Vec<Identifier>,
+    pub join_criteria: Option<JoinCriteria>,
     pub join_data_type: Option<JoinDataType>,
 }
 
@@ -545,7 +679,7 @@ pub struct Unpivot {
     /// When `ids` is [None] (for SQL statements), all remaining columns are included.
     /// When `ids` is [Some] (for the DataFrame API), only the specified columns are included.
     pub ids: Option<Vec<Expr>>,
-    pub values: Vec<UnpivotValue>,
+    pub values: Option<Vec<UnpivotValue>>,
     pub variable_column_name: Identifier,
     pub value_column_names: Vec<Identifier>,
     pub include_nulls: bool,
@@ -576,9 +710,10 @@ pub struct GroupMap {
     pub sorting_expressions: Vec<Expr>,
     pub initial_input: Option<Box<QueryPlan>>,
     pub initial_grouping_expressions: Vec<Expr>,
-    pub is_map_groups_with_state: Option<bool>, // TODO: this should probably be an enum
-    pub output_mode: Option<String>,            // TODO: this should probably be an enum
-    pub timeout_conf: Option<String>,           // TODO: this should probably be a struct
+    pub is_map_groups_with_state: Option<bool>,
+    pub output_mode: Option<String>,
+    pub timeout_conf: Option<String>,
+    pub state_schema: Option<Schema>,
 }
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
@@ -609,8 +744,8 @@ pub struct ApplyInPandasWithState {
     pub function: CommonInlineUserDefinedFunction,
     pub output_schema: Schema,
     pub state_schema: Schema,
-    pub output_mode: String,  // TODO: this should probably be an enum
-    pub timeout_conf: String, // TODO: this should probably be a struct
+    pub output_mode: String,
+    pub timeout_conf: String,
 }
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
@@ -624,23 +759,33 @@ pub struct HtmlString {
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct TableDefinition {
-    pub schema: Schema,
+    pub columns: Vec<TableColumnDefinition>,
     pub comment: Option<String>,
-    pub column_defaults: Vec<(String, Expr)>,
     pub constraints: Vec<TableConstraint>,
     pub location: Option<String>,
-    pub serde_properties: Vec<(String, String)>,
     pub file_format: Option<TableFileFormat>,
     pub row_format: Option<TableRowFormat>,
-    pub table_partition_cols: Vec<Identifier>,
-    pub file_sort_order: Vec<Vec<SortOrder>>,
+    pub partition_by: Vec<Identifier>,
+    pub sort_by: Vec<SortOrder>,
+    pub bucket_by: Option<SaveBucketBy>,
+    pub cluster_by: Vec<ObjectName>,
     pub if_not_exists: bool,
-    pub or_replace: bool,
-    pub unbounded: bool,
+    pub replace: bool,
     pub options: Vec<(String, String)>,
-    /// The query for `CREATE TABLE ... AS SELECT ...` (CTAS) statements.
-    pub query: Option<Box<QueryPlan>>,
-    pub definition: Option<String>,
+    pub properties: Vec<(String, String)>,
+}
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct TableColumnDefinition {
+    pub name: String,
+    pub data_type: DataType,
+    pub nullable: bool,
+    pub comment: Option<String>,
+    /// An optional SQL expression string for the default value.
+    pub default: Option<String>,
+    /// An optional SQL expression string to calculate the generated value.
+    pub generated_always_as: Option<String>,
 }
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
@@ -660,39 +805,46 @@ pub struct CatalogDefinition {
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
-pub enum ViewKind {
-    /// A view that is stored in the catalog.
-    Default,
-    /// A temporary view that is tied to the session.
-    Temporary,
-    /// A temporary view that is available for all sessions in the same Spark application.
-    /// See also: <https://spark.apache.org/docs/latest/sql-getting-started.html#global-temporary-view>
-    GlobalTemporary,
+pub struct ViewDefinition {
+    /// The corresponding SQL query that defines the view.
+    pub definition: String,
+    pub columns: Option<Vec<ViewColumnDefinition>>,
+    pub if_not_exists: bool,
+    pub replace: bool,
+    pub comment: Option<String>,
+    pub properties: Vec<(String, String)>,
 }
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
-pub struct ViewDefinition {
+pub struct ViewColumnDefinition {
+    pub name: String,
+    pub comment: Option<String>,
+}
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct TemporaryViewDefinition {
     pub input: Box<QueryPlan>,
-    pub columns: Option<Vec<Identifier>>,
-    pub kind: ViewKind,
+    pub columns: Option<Vec<ViewColumnDefinition>>,
+    pub if_not_exists: bool,
     pub replace: bool,
-    pub definition: Option<String>,
+    pub comment: Option<String>,
+    pub properties: Vec<(String, String)>,
 }
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct Write {
     pub input: Box<QueryPlan>,
-    pub source: Option<String>, // TODO: is this the same as "provider" in `WriteOperationV2`?
+    pub source: Option<String>,
     pub save_type: SaveType,
-    pub mode: SaveMode,
-    pub sort_columns: Vec<Identifier>,
+    pub mode: Option<SaveMode>,
+    pub sort_columns: Vec<SortOrder>,
     pub partitioning_columns: Vec<Identifier>,
+    pub clustering_columns: Vec<Identifier>,
     pub bucket_by: Option<SaveBucketBy>,
     pub options: Vec<(String, String)>,
-    pub table_properties: Vec<(String, String)>,
-    pub overwrite_condition: Option<Expr>,
 }
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
@@ -719,21 +871,79 @@ pub struct SaveBucketBy {
     pub num_buckets: usize,
 }
 
-#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[derive(Debug, Default, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub enum SaveMode {
-    // TODO: understand the difference between save modes in
-    //  `WriteOperation` and `WriteOperationV2`
+    Append,
+    Overwrite,
+    #[default]
+    ErrorIfExists,
+    IgnoreIfExists,
+}
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct WriteTo {
+    pub input: Box<QueryPlan>,
+    pub provider: Option<String>,
+    pub table: ObjectName,
+    pub mode: WriteToMode,
+    pub partitioning_columns: Vec<Expr>,
+    pub clustering_columns: Vec<Identifier>,
+    pub options: Vec<(String, String)>,
+    pub table_properties: Vec<(String, String)>,
+}
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub enum WriteToMode {
     Append,
     Create,
     CreateOrReplace,
-    ErrorIfExists,
-    Ignore,
-    Overwrite,
+    Overwrite { condition: Box<Expr> },
     OverwritePartitions,
     Replace,
 }
 
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct WriteStream {
+    pub input: Box<QueryPlan>,
+    pub format: String,
+    pub options: Vec<(String, String)>,
+    pub partitioning_column_names: Vec<Identifier>,
+    pub query_name: String,
+    pub foreach_writer: Option<FunctionDefinition>,
+    pub foreach_batch: Option<FunctionDefinition>,
+    pub clustering_column_names: Vec<Identifier>,
+    pub sink_destination: Option<WriteStreamSinkDestination>,
+}
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase", rename_all_fields = "camelCase")]
+pub enum WriteStreamSinkDestination {
+    Path { path: String },
+    Table { table: ObjectName },
+}
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase", rename_all_fields = "camelCase")]
+pub enum InsertMode {
+    InsertByPosition {
+        overwrite: bool,
+    },
+    InsertByName {
+        overwrite: bool,
+    },
+    InsertByColumns {
+        columns: Vec<Identifier>,
+        overwrite: bool,
+    },
+    Replace {
+        condition: Box<Expr>,
+    },
+}
+
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub enum JoinType {
@@ -741,11 +951,21 @@ pub enum JoinType {
     FullOuter,
     LeftOuter,
     RightOuter,
-    LeftAnti,
     LeftSemi,
+    RightSemi,
+    LeftAnti,
+    RightAnti,
     Cross,
 }
 
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase", rename_all_fields = "camelCase")]
+pub enum JoinCriteria {
+    Natural,
+    On(Expr),
+    Using(Vec<Identifier>),
+}
+
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub struct JoinDataType {
@@ -793,6 +1013,107 @@ pub struct StorageLevel {
     pub replication: usize,
 }
 
+impl FromStr for StorageLevel {
+    type Err = CommonError;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        match s.to_ascii_uppercase().as_str() {
+            "NONE" => Ok(Self {
+                use_disk: false,
+                use_memory: false,
+                use_off_heap: false,
+                deserialized: false,
+                replication: 1,
+            }),
+            "DISK_ONLY" => Ok(Self {
+                use_disk: true,
+                use_memory: false,
+                use_off_heap: false,
+                deserialized: false,
+                replication: 1,
+            }),
+            "DISK_ONLY_2" => Ok(Self {
+                use_disk: true,
+                use_memory: false,
+                use_off_heap: false,
+                deserialized: false,
+                replication: 2,
+            }),
+            "DISK_ONLY_3" => Ok(Self {
+                use_disk: true,
+                use_memory: false,
+                use_off_heap: false,
+                deserialized: false,
+                replication: 3,
+            }),
+            "MEMORY_ONLY" => Ok(Self {
+                use_disk: false,
+                use_memory: true,
+                use_off_heap: false,
+                deserialized: true,
+                replication: 1,
+            }),
+            "MEMORY_ONLY_2" => Ok(Self {
+                use_disk: false,
+                use_memory: true,
+                use_off_heap: false,
+                deserialized: true,
+                replication: 2,
+            }),
+            "MEMORY_ONLY_SER" => Ok(Self {
+                use_disk: false,
+                use_memory: true,
+                use_off_heap: false,
+                deserialized: false,
+                replication: 1,
+            }),
+            "MEMORY_ONLY_SER_2" => Ok(Self {
+                use_disk: false,
+                use_memory: true,
+                use_off_heap: false,
+                deserialized: false,
+                replication: 2,
+            }),
+            "MEMORY_AND_DISK" => Ok(Self {
+                use_disk: true,
+                use_memory: true,
+                use_off_heap: false,
+                deserialized: true,
+                replication: 1,
+            }),
+            "MEMORY_AND_DISK_2" => Ok(Self {
+                use_disk: true,
+                use_memory: true,
+                use_off_heap: false,
+                deserialized: true,
+                replication: 2,
+            }),
+            "MEMORY_AND_DISK_SER" => Ok(Self {
+                use_disk: true,
+                use_memory: true,
+                use_off_heap: false,
+                deserialized: false,
+                replication: 1,
+            }),
+            "MEMORY_AND_DISK_SER_2" => Ok(Self {
+                use_disk: true,
+                use_memory: true,
+                use_off_heap: false,
+                deserialized: false,
+                replication: 2,
+            }),
+            "OFF_HEAP" => Ok(Self {
+                use_disk: true,
+                use_memory: true,
+                use_off_heap: true,
+                deserialized: false,
+                replication: 1,
+            }),
+            _ => Err(CommonError::invalid(format!("storage level: {s}"))),
+        }
+    }
+}
+
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub enum TableConstraint {
@@ -808,23 +1129,31 @@ pub enum TableConstraint {
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
-pub struct TableFileFormat {
-    pub input_format: String,
-    pub output_format: Option<String>, // if None: INPUTFORMAT AND OUTPUTFORMAT not specified.
+pub enum TableFileFormat {
+    General {
+        format: String,
+    },
+    Table {
+        input_format: String,
+        output_format: String,
+    },
 }
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
 pub enum TableRowFormat {
-    Serde(String),
-    Delimited(Vec<TableRowDelimiter>),
-}
-
-#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
-#[serde(rename_all = "camelCase")]
-pub struct TableRowDelimiter {
-    pub delimiter: String,
-    pub char: Identifier,
+    Serde {
+        name: String,
+        properties: Vec<(String, String)>,
+    },
+    Delimited {
+        fields_terminated_by: Option<String>,
+        fields_escaped_by: Option<String>,
+        collection_items_terminated_by: Option<String>,
+        map_keys_terminated_by: Option<String>,
+        lines_terminated_by: Option<String>,
+        null_defined_as: Option<String>,
+    },
 }
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
@@ -842,7 +1171,22 @@ pub enum ExplainMode {
 
 #[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
 #[serde(rename_all = "camelCase")]
-pub struct AlterTableLocation {
-    pub location: Identifier,
-    pub set_location: bool,
+pub enum AlterTableOperation {
+    Unknown,
+    // TODO: add all the alter table operations
+}
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub enum AlterViewOperation {
+    Unknown,
+    // TODO: add all the alter view operations
+}
+
+#[derive(Debug, Clone, PartialEq, Serialize, Deserialize)]
+#[serde(rename_all = "camelCase")]
+pub struct Delete {
+    pub table: ObjectName,
+    pub table_alias: Option<Identifier>,
+    pub condition: Option<Expr>,
 }
