@@ -1,7 +1,7 @@
-use crate::function::common::Function;
+use crate::function::common::ScalarFunction;
 
-pub(super) fn list_built_in_lambda_functions() -> Vec<(&'static str, Function)> {
-    use crate::function::common::FunctionBuilder as F;
+pub(super) fn list_built_in_lambda_functions() -> Vec<(&'static str, ScalarFunction)> {
+    use crate::function::common::ScalarFunctionBuilder as F;
 
     vec![
         ("aggregate", F::unknown("aggregate")),
