@@ -1,4 +1,4 @@
-use crate::function::common::Function;
+use crate::function::common::ScalarFunction;
 
 mod array;
 mod bitwise;
@@ -17,9 +17,10 @@ mod predicate;
 mod string;
 mod r#struct;
 mod url;
+mod variant;
 mod xml;
 
-pub(super) fn list_built_in_scalar_functions() -> Vec<(&'static str, Function)> {
+pub(super) fn list_built_in_scalar_functions() -> Vec<(&'static str, ScalarFunction)> {
     let mut output = Vec::new();
     output.extend(array::list_built_in_array_functions());
     output.extend(bitwise::list_built_in_bitwise_functions());
@@ -38,6 +39,7 @@ pub(super) fn list_built_in_scalar_functions() -> Vec<(&'static str, Function)>
     output.extend(string::list_built_in_string_functions());
     output.extend(r#struct::list_built_in_struct_functions());
     output.extend(url::list_built_in_url_functions());
+    output.extend(variant::list_built_in_variant_functions());
     output.extend(xml::list_built_in_xml_functions());
     output
 }
