@@ -1,9 +1,12 @@
-use crate::function::common::Function;
+use crate::function::common::ScalarFunction;
 
-pub(super) fn list_built_in_xml_functions() -> Vec<(&'static str, Function)> {
-    use crate::function::common::FunctionBuilder as F;
+pub(super) fn list_built_in_xml_functions() -> Vec<(&'static str, ScalarFunction)> {
+    use crate::function::common::ScalarFunctionBuilder as F;
 
     vec![
+        ("from_xml", F::unknown("from_xml")),
+        ("schema_of_xml", F::unknown("schema_of_xml")),
+        ("to_xml", F::unknown("to_xml")),
         ("xpath", F::unknown("xpath")),
         ("xpath_boolean", F::unknown("xpath_boolean")),
         ("xpath_double", F::unknown("xpath_double")),
