@@ -1,3 +1,4 @@
+mod cli;
 mod spark;
 
 use pyo3::prelude::*;
@@ -5,6 +6,7 @@ use pyo3::prelude::*;
 #[pymodule]
 fn _native(m: &Bound<'_, PyModule>) -> PyResult<()> {
     spark::register_module(m)?;
+    m.add_function(wrap_pyfunction!(cli::main, m)?)?;
     m.add("_SAIL_VERSION", env!("CARGO_PKG_VERSION"))?;
     Ok(())
 }
