@@ -1,8 +1,3 @@
-use mimalloc::MiMalloc;
-
-#[global_allocator]
-static GLOBAL: MiMalloc = MiMalloc;
-
 mod config;
 mod debug;
 pub mod entrypoint;
@@ -13,8 +8,12 @@ mod schema;
 pub mod server;
 mod service;
 mod session;
+mod session_manager;
+mod streaming;
 
-const SPARK_VERSION: &str = "3.5.1";
+pub use proto::data_type_json::JsonDataType;
+
+const SPARK_VERSION: &str = "4.0.0";
 
 pub mod spark {
     #[allow(clippy::all)]
