@@ -1,13 +1,18 @@
 use std::collections::HashMap;
 
 use sail_common::spec;
-use sail_sql::data_type::parse_data_type;
-use sail_sql::expression::common::{
-    parse_expression, parse_object_name, parse_qualified_wildcard, parse_wildcard_expression,
+use sail_sql_analyzer::data_type::from_ast_data_type;
+use sail_sql_analyzer::expression::{
+    from_ast_expression, from_ast_object_name, from_ast_qualified_wildcard,
 };
+use sail_sql_analyzer::parser::{
+    parse_data_type, parse_expression, parse_named_expression, parse_object_name,
+    parse_qualified_wildcard,
+};
+use sail_sql_analyzer::query::from_ast_named_expression;
 
 use crate::error::{ProtoFieldExt, SparkError, SparkResult};
-use crate::spark::connect::expression::cast::CastToType;
+use crate::spark::connect::expression::cast::{CastToType, EvalMode};
 use crate::spark::connect::expression::sort_order::{NullOrdering, SortDirection};
 use crate::spark::connect::expression::window::window_frame::frame_boundary::Boundary;
 use crate::spark::connect::expression::window::window_frame::{FrameBoundary, FrameType};
@@ -27,47 +32,70 @@ impl TryFrom<Expression> for spec::Expr {
     type Error = SparkError;
 
     fn try_from(expr: Expression) -> SparkResult<spec::Expr> {
-        let Expression { expr_type } = expr;
+        let Expression {
+            common: _,
+            expr_type,
+        } = expr;
         let expr_type = expr_type.required("expression type")?;
         match expr_type {
             ExprType::Literal(literal) => Ok(spec::Expr::Literal(literal.try_into()?)),
             ExprType::UnresolvedAttribute(UnresolvedAttribute {
                 unparsed_identifier,
                 plan_id,
+                is_metadata_column,
             }) => {
-                // TODO: Revisit heuristic for parsing object names.
-                let name = if unparsed_identifier.contains('.') {
-                    parse_object_name(unparsed_identifier.as_str())?
-                } else {
-                    spec::ObjectName::new_unqualified(unparsed_identifier.into())
-                };
-                Ok(spec::Expr::UnresolvedAttribute { name, plan_id })
+                // The unparsed identifier such as `a.b` is supposed to be parsed as nested
+                // object names. However, there may be raw identifier such as `array(1)` which
+                // cannot be parsed. Therefore, when parsing fails, we create an object name
+                // containing the single raw identifier.
+                let name = parse_object_name(unparsed_identifier.as_str())
+                    .and_then(from_ast_object_name)
+                    .unwrap_or_else(|_| spec::ObjectName::from(vec![unparsed_identifier]));
+                Ok(spec::Expr::UnresolvedAttribute {
+                    name,
+                    plan_id,
+                    is_metadata_column: is_metadata_column.unwrap_or(false),
+                })
             }
             ExprType::UnresolvedFunction(UnresolvedFunction {
                 function_name,
                 arguments,
                 is_distinct,
                 is_user_defined_function,
-            }) => Ok(spec::Expr::UnresolvedFunction {
-                function_name,
+                is_internal,
+            }) => Ok(spec::Expr::UnresolvedFunction(spec::UnresolvedFunction {
+                function_name: spec::ObjectName::bare(function_name),
                 arguments: arguments
                     .into_iter()
                     .map(|x| x.try_into())
                     .collect::<SparkResult<_>>()?,
+                named_arguments: vec![],
                 is_distinct,
                 is_user_defined_function,
-            }),
+                is_internal,
+                ignore_nulls: None,
+                filter: None,
+                order_by: None,
+            })),
             ExprType::ExpressionString(ExpressionString { expression }) => {
                 let expr = parse_expression(expression.as_str())
-                    .or_else(|_| parse_wildcard_expression(expression.as_str()))?;
+                    .and_then(from_ast_expression)
+                    .or_else(|_| {
+                        parse_named_expression(expression.as_str())
+                            .and_then(from_ast_named_expression)
+                    })?;
                 Ok(expr)
             }
-            ExprType::UnresolvedStar(UnresolvedStar { unparsed_target }) => {
+            ExprType::UnresolvedStar(UnresolvedStar {
+                unparsed_target,
+                plan_id,
+            }) => {
                 let target = unparsed_target
-                    .map(|x| parse_qualified_wildcard(x.as_str()))
+                    .map(|x| from_ast_qualified_wildcard(parse_qualified_wildcard(x.as_str())?))
                     .transpose()?;
                 Ok(spec::Expr::UnresolvedStar {
                     target,
+                    plan_id,
                     wildcard_options: Default::default(),
                 })
             }
@@ -80,17 +108,19 @@ impl TryFrom<Expression> for spec::Expr {
                 let expr = expr.required("alias expression")?;
                 let metadata: Option<HashMap<String, String>> = metadata
                     .map(|x| {
-                        serde_json::from_str(&x).map_err(SparkError::from).and_then(
-                            |x: serde_json::Value| {
+                        serde_json::from_str(&x)
+                            .map_err(SparkError::from)
+                            .and_then(|x: serde_json::Value| {
                                 x.as_object()
                                     .ok_or_else(|| SparkError::invalid("alias metadata"))
                                     .map(|x| {
                                         x.into_iter()
-                                            .map(|(k, v)| (k.clone(), v.to_string()))
-                                            .collect()
+                                            .map(|(k, v)| {
+                                                Ok((k.clone(), serde_json::to_string(v)?))
+                                            })
+                                            .collect::<SparkResult<_>>()
                                     })
-                            },
-                        )
+                            })?
                     })
                     .transpose()?;
                 let name: Vec<spec::Identifier> = name.into_iter().map(|x| x.into()).collect();
@@ -101,16 +131,22 @@ impl TryFrom<Expression> for spec::Expr {
                 })
             }
             ExprType::Cast(cast) => {
-                let Cast { expr, cast_to_type } = *cast;
+                let Cast {
+                    expr,
+                    eval_mode, // TODO: support eval mode
+                    cast_to_type,
+                } = *cast;
                 let expr = expr.required("cast expression")?;
                 let cast_to_type = cast_to_type.required("cast type")?;
                 let cast_to_type = match cast_to_type {
                     CastToType::Type(x) => x.try_into()?,
-                    CastToType::TypeStr(s) => parse_data_type(s.as_str())?,
+                    CastToType::TypeStr(s) => from_ast_data_type(parse_data_type(s.as_str())?)?,
                 };
                 Ok(spec::Expr::Cast {
                     expr: Box::new((*expr).try_into()?),
                     cast_to_type,
+                    rename: false,
+                    is_try: matches!(EvalMode::try_from(eval_mode), Ok(EvalMode::Try)),
                 })
             }
             ExprType::UnresolvedRegex(UnresolvedRegex { col_name, plan_id }) => {
@@ -141,15 +177,18 @@ impl TryFrom<Expression> for spec::Expr {
                 let window_function = window_function.required("window function")?;
                 Ok(spec::Expr::Window {
                     window_function: Box::new((*window_function).try_into()?),
-                    partition_spec: partition_spec
-                        .into_iter()
-                        .map(|x| x.try_into())
-                        .collect::<SparkResult<_>>()?,
-                    order_spec: order_spec
-                        .into_iter()
-                        .map(|x| x.try_into())
-                        .collect::<SparkResult<_>>()?,
-                    frame_spec: frame_spec.map(|x| (*x).try_into()).transpose()?,
+                    window: spec::Window::Unnamed {
+                        cluster_by: vec![],
+                        partition_by: partition_spec
+                            .into_iter()
+                            .map(|x| x.try_into())
+                            .collect::<SparkResult<_>>()?,
+                        order_by: order_spec
+                            .into_iter()
+                            .map(|x| x.try_into())
+                            .collect::<SparkResult<_>>()?,
+                        frame: frame_spec.map(|x| (*x).try_into()).transpose()?,
+                    },
                 })
             }
             ExprType::UnresolvedExtractValue(extract) => {
@@ -170,7 +209,7 @@ impl TryFrom<Expression> for spec::Expr {
                 let struct_expression = struct_expression.required("struct expression")?;
                 Ok(spec::Expr::UpdateFields {
                     struct_expression: Box::new((*struct_expression).try_into()?),
-                    field_name: parse_object_name(field_name.as_str())?,
+                    field_name: from_ast_object_name(parse_object_name(field_name.as_str())?)?,
                     value_expression: value_expression
                         .map(|x| -> SparkResult<_> { Ok(Box::new((*x).try_into()?)) })
                         .transpose()?,
@@ -185,13 +224,25 @@ impl TryFrom<Expression> for spec::Expr {
             ExprType::CallFunction(CallFunction {
                 function_name,
                 arguments,
-            }) => Ok(spec::Expr::CallFunction {
-                function_name,
-                arguments: arguments
-                    .into_iter()
-                    .map(|x| x.try_into())
-                    .collect::<SparkResult<_>>()?,
-            }),
+            }) => {
+                let function_name =
+                    from_ast_object_name(parse_object_name(function_name.as_str())?)?;
+                Ok(spec::Expr::CallFunction {
+                    function_name,
+                    arguments: arguments
+                        .into_iter()
+                        .map(|x| x.try_into())
+                        .collect::<SparkResult<_>>()?,
+                })
+            }
+            ExprType::NamedArgumentExpression(_) => {
+                Err(SparkError::todo("named argument expression"))
+            }
+            ExprType::MergeAction(_) => Err(SparkError::todo("merge action expression")),
+            ExprType::TypedAggregateExpression(_) => {
+                Err(SparkError::todo("typed aggregate expression"))
+            }
+            ExprType::SubqueryExpression(_) => Err(SparkError::todo("subquery expression")),
             ExprType::Extension(_) => Err(SparkError::todo("extension expression")),
         }
     }
@@ -245,17 +296,24 @@ impl TryFrom<WindowFrame> for spec::WindowFrame {
     type Error = SparkError;
 
     fn try_from(window_frame: WindowFrame) -> SparkResult<spec::WindowFrame> {
+        fn boundary(boundary: Option<Box<FrameBoundary>>) -> Option<Boundary> {
+            boundary.and_then(|x| {
+                let FrameBoundary { boundary } = *x;
+                boundary
+            })
+        }
+
         let WindowFrame {
             frame_type,
             lower,
             upper,
         } = window_frame;
         let frame_type = FrameType::try_from(frame_type)?;
-        let lower = lower.required("lower window frame boundary")?;
-        let upper = upper.required("upper window frame boundary")?;
+        let lower = boundary(lower).required("lower window frame boundary")?;
+        let upper = boundary(upper).required("upper window frame boundary")?;
         let frame_type = frame_type.try_into()?;
-        let lower = (*lower).try_into()?;
-        let upper = (*upper).try_into()?;
+        let lower = WindowBoundaryKind::Lower(lower).try_into()?;
+        let upper = WindowBoundaryKind::Upper(upper).try_into()?;
         Ok(spec::WindowFrame {
             frame_type,
             lower,
@@ -276,19 +334,34 @@ impl TryFrom<FrameType> for spec::WindowFrameType {
     }
 }
 
-impl TryFrom<FrameBoundary> for spec::WindowFrameBoundary {
+enum WindowBoundaryKind {
+    Lower(Boundary),
+    Upper(Boundary),
+}
+
+impl TryFrom<WindowBoundaryKind> for spec::WindowFrameBoundary {
     type Error = SparkError;
 
-    fn try_from(frame_boundary: FrameBoundary) -> SparkResult<spec::WindowFrameBoundary> {
-        let FrameBoundary { boundary } = frame_boundary;
-        let boundary = boundary.required("window frame boundary")?;
-        match boundary {
-            Boundary::CurrentRow(true) => Ok(spec::WindowFrameBoundary::CurrentRow),
-            Boundary::Unbounded(true) => Ok(spec::WindowFrameBoundary::Unbounded),
-            Boundary::Value(expr) => Ok(spec::WindowFrameBoundary::Value(Box::new(
-                (*expr).try_into()?,
-            ))),
-            Boundary::CurrentRow(false) | Boundary::Unbounded(false) => {
+    fn try_from(kind: WindowBoundaryKind) -> SparkResult<spec::WindowFrameBoundary> {
+        match kind {
+            WindowBoundaryKind::Lower(Boundary::CurrentRow(true))
+            | WindowBoundaryKind::Upper(Boundary::CurrentRow(true)) => {
+                Ok(spec::WindowFrameBoundary::CurrentRow)
+            }
+            WindowBoundaryKind::Lower(Boundary::Unbounded(true)) => {
+                Ok(spec::WindowFrameBoundary::UnboundedPreceding)
+            }
+            WindowBoundaryKind::Upper(Boundary::Unbounded(true)) => {
+                Ok(spec::WindowFrameBoundary::UnboundedFollowing)
+            }
+            WindowBoundaryKind::Lower(Boundary::Value(expr))
+            | WindowBoundaryKind::Upper(Boundary::Value(expr)) => Ok(
+                spec::WindowFrameBoundary::Value(Box::new((*expr).try_into()?)),
+            ),
+            WindowBoundaryKind::Lower(Boundary::CurrentRow(false))
+            | WindowBoundaryKind::Lower(Boundary::Unbounded(false))
+            | WindowBoundaryKind::Upper(Boundary::CurrentRow(false))
+            | WindowBoundaryKind::Upper(Boundary::Unbounded(false)) => {
                 Err(SparkError::invalid("invalid window frame boundary"))
             }
         }
@@ -305,12 +378,14 @@ impl TryFrom<CommonInlineUserDefinedFunction> for spec::CommonInlineUserDefinedF
             function_name,
             deterministic,
             arguments,
+            is_distinct,
             function,
         } = function;
         let function = function.required("common inline UDF function")?;
         Ok(spec::CommonInlineUserDefinedFunction {
-            function_name,
+            function_name: function_name.into(),
             deterministic,
+            is_distinct,
             arguments: arguments
                 .into_iter()
                 .map(|x| x.try_into())
@@ -332,6 +407,7 @@ impl TryFrom<udf::Function> for spec::FunctionDefinition {
                 eval_type,
                 command,
                 python_ver,
+                additional_includes,
             }) => {
                 let output_type = output_type.required("Python UDF output type")?;
                 Ok(spec::FunctionDefinition::PythonUdf {
@@ -339,6 +415,7 @@ impl TryFrom<udf::Function> for spec::FunctionDefinition {
                     eval_type: spec::PySparkUdfType::try_from(eval_type)?,
                     command,
                     python_version: python_ver,
+                    additional_includes,
                 })
             }
             Function::ScalarScalaUdf(ScalarScalaUdf {
@@ -346,6 +423,7 @@ impl TryFrom<udf::Function> for spec::FunctionDefinition {
                 input_types,
                 output_type,
                 nullable,
+                aggregate,
             }) => {
                 let output_type = output_type.required("Scalar Scala UDF output type")?;
                 Ok(spec::FunctionDefinition::ScalarScalaUdf {
@@ -356,6 +434,7 @@ impl TryFrom<udf::Function> for spec::FunctionDefinition {
                         .collect::<SparkResult<_>>()?,
                     output_type: output_type.try_into()?,
                     nullable,
+                    aggregate,
                 })
             }
             Function::JavaUdf(JavaUdf {
@@ -385,7 +464,7 @@ impl TryFrom<CommonInlineUserDefinedTableFunction> for spec::CommonInlineUserDef
         } = function;
         let function = function.required("common inline UDTF function")?;
         Ok(spec::CommonInlineUserDefinedTableFunction {
-            function_name,
+            function_name: function_name.into(),
             deterministic,
             arguments: arguments
                 .into_iter()
@@ -440,7 +519,8 @@ mod tests {
 
     use sail_common::spec;
     use sail_common::tests::test_gold_set;
-    use sail_sql::expression::common::parse_wildcard_expression;
+    use sail_sql_analyzer::parser::parse_named_expression;
+    use sail_sql_analyzer::query::from_ast_named_expression;
 
     use crate::error::SparkError;
 
@@ -448,16 +528,16 @@ mod tests {
     fn test_sql_to_expression() -> Result<(), Box<dyn std::error::Error>> {
         // Run the test in a separate thread with a large stack size
         // so that it can handle deeply nested expressions.
-        let builder = thread::Builder::new().stack_size(128 * 1024 * 1024);
+        let builder = thread::Builder::new().stack_size(160 * 1024 * 1024);
         let handle = builder.spawn(|| {
             test_gold_set(
                 "tests/gold_data/expression/*.json",
                 |sql: String| {
-                    let expr = parse_wildcard_expression(&sql)?;
+                    let expr = from_ast_named_expression(parse_named_expression(&sql)?)?;
                     if sql.len() > 128 {
-                        Ok(spec::Expr::Literal(spec::Literal::String(
-                            "Result omitted for long expression.".to_string(),
-                        )))
+                        Ok(spec::Expr::Literal(spec::Literal::Utf8 {
+                            value: Some("Result omitted for long expression.".to_string()),
+                        }))
                     } else {
                         Ok(expr)
                     }
